@@ -361,7 +361,9 @@ public:
    * Get root index of opened directory in directory proxy model.
    * @return index of directory root.
    */
-  QModelIndex getDirRootIndex() const { return m_dirProxyModelRootIndex; }
+  QPersistentModelIndex getDirRootIndex() const {
+    return m_dirProxyModelRootIndex;
+  }
 
   /**
    * Get directory path of opened directory.
@@ -371,7 +373,7 @@ public:
 
   /**
    * Save all changed files.
-   * saveStarted() and saveProgress() are emitted while saving files.
+   * longRunningOperationProgress() is emitted while saving files.
    *
    * @return list of files with error, empty if ok.
    */
@@ -1105,18 +1107,6 @@ signals:
    */
   void confirmedOpenDirectoryRequested(const QStringList& paths);
 
-  /**
-   * Emitted when saving files is started.
-   * @param totalFiles total number of files to be saved
-   */
-  void saveStarted(int totalFiles);
-
-  /**
-   * Emitted when a file has bee saved.
-   * @param numFiles number of files saved
-   */
-  void saveProgress(int numFiles);
-
   /**
    * Emitted before an operation on the selected files is performed.
    * The GUI should update the files of the current selection when
@@ -1165,8 +1155,10 @@ signals:
    * Emitted when a file is filtered.
    * @param type filter event type, enum FileFilter::FilterEventType
    * @param fileName name of filtered file
+   * @param passed number of files which passed the filter
+   * @param total total number of files checked
    */
-  void fileFiltered(int type, const QString& fileName);
+  void fileFiltered(int type, const QString& fileName, int passed, int total);
 
   /**
    * Emitted before an audio file is played.
@@ -1215,7 +1207,28 @@ signals:
    */
   void frameEditorChanged();
 
+  /**
+   * Emitted to report progress about a long running operation.
+   *
+   * This signal is used to report different states of a long running operation.
+   * - Operation is started: done is -1,
+   * - Operation is finished: done == total and total is not 0,
+   * - Operation in progress: done < total or both done and total are 0
+   *
+   * @param name name of operation
+   * @param done amount of work done
+   * @param total total amount of work
+   * @param abort if not 0, can be set to true to abort the operation
+   */
+  void longRunningOperationProgress(const QString& name, int done, int total,
+                                    bool* abort);
+
 private slots:
+  /**
+   * Apply file filter after the file system model has been reset.
+   */
+  void applyFilterAfterReset();
+
   /**
    * Apply single file to file filter.
    *
@@ -1288,6 +1301,25 @@ private:
    */
   void addFrame(Frame::TagNumber tagNr, const Frame* frame, bool edit = false);
 
+  /**
+   * Open directory after resetting the file system model.
+   * This will create a new file system model and reset the file and directory
+   * proxy models.
+   * When finished directoryOpened() is emitted, also if false is returned.
+   *
+   * @param paths file or directory paths, if multiple paths are given, the
+   * common directory is opened and the files are selected, if empty, the
+   * currently open directory is reopened
+   *
+   * @return true if ok.
+   */
+  bool openDirectoryAfterReset(const QStringList& paths = QStringList());
+
+  /**
+   * Second stage for applyFilter().
+   */
+  void proceedApplyingFilter();
+
   /**
    * Set the coverArtImageId property to a new value.
    * This can be used to trigger an update of QML images.
@@ -1348,7 +1380,7 @@ private:
   /** Root index in file proxy model */
   QPersistentModelIndex m_fileProxyModelRootIndex;
   /** Root index in directory proxy model */
-  QModelIndex m_dirProxyModelRootIndex;
+  QPersistentModelIndex m_dirProxyModelRootIndex;
   /** Indexes of opened file in file proxy model */
   QList<QPersistentModelIndex> m_fileProxyModelFileIndexes;
   /** Importers for different servers */
@@ -1367,6 +1399,8 @@ private:
   /* Context for filterNextFile() */
   FileFilter* m_fileFilter;
   QString m_lastProcessedDirName;
+  int m_filterPassed;
+  int m_filterTotal;
   /* Context for batchImportNextFile() */
   BatchImportProfile* m_namedBatchImportProfile;
   const BatchImportProfile* m_batchImportProfile;
@@ -1390,6 +1424,8 @@ private:
 
   /** true if list is filtered */
   bool m_filtered;
+  /** true if a selection operation is running */
+  bool m_selectionOperationRunning;
 
   /** Fallback for path to search for plugins */
   static QString s_pluginsPathFallback;
