@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 9 Jan 2003
  *
- * Copyright (C) 2003-2013  Urs Fleisch
+ * Copyright (C) 2003-2017  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -30,6 +30,7 @@
 #include <QMessageBox>
 #include <QInputDialog>
 #include <QProgressBar>
+#include <QToolButton>
 #include <QCloseEvent>
 #include <QHBoxLayout>
 #include <QVBoxLayout>
@@ -56,6 +57,7 @@
 #include "downloaddialog.h"
 #include "playlistdialog.h"
 #include "editframefieldsdialog.h"
+#include "progresswidget.h"
 #include "fileproxymodel.h"
 #include "fileproxymodeliterator.h"
 #include "modeliterator.h"
@@ -75,7 +77,6 @@
 #include "batchimporter.h"
 #include "dirrenamer.h"
 #include "iplatformtools.h"
-#include "qtcompatmac.h"
 #include "saferename.h"
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
 #include "audioplayer.h"
@@ -100,11 +101,14 @@ BaseMainWindowImpl::BaseMainWindowImpl(QMainWindow* mainWin,
   m_exportDialog(0), m_findReplaceDialog(0), m_renDirDialog(0),
   m_numberTracksDialog(0), m_filterDialog(0),
   m_downloadDialog(new DownloadDialog(m_w, tr("Download"))),
-  m_playlistDialog(0), m_progressDialog(0), m_editFrameDialog(0),
+  m_playlistDialog(0), m_progressWidget(0),
+  m_progressBar(0), m_progressAbortButton(0), m_editFrameDialog(0),
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
   m_playToolBar(0),
 #endif
   m_editFrameTaggedFile(0), m_editFrameTagNr(Frame::Tag_2),
+  m_progressTerminationHandler(0),
+  m_progressDisconnected(false),
   m_findReplaceActive(false), m_expandNotificationNeeded(false)
 {
   m_downloadDialog->close();
@@ -142,6 +146,8 @@ BaseMainWindowImpl::BaseMainWindowImpl(QMainWindow* mainWin,
           this, SLOT(updateWindowCaption()));
   connect(m_app, SIGNAL(filteredChanged(bool)),
           this, SLOT(updateWindowCaption()));
+  connect(m_app, SIGNAL(longRunningOperationProgress(QString,int,int,bool*)),
+          this, SLOT(showOperationProgress(QString,int,int,bool*)));
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
   connect(m_app, SIGNAL(aboutToPlayAudio()), this, SLOT(showPlayToolBar()));
 #endif
@@ -230,6 +236,62 @@ void BaseMainWindowImpl::readOptions()
   m_form->readConfig();
 }
 
+/**
+ * Show progress of long running operation in status bar.
+ * @param name name of operation
+ * @param done amount of work done
+ * @param total total amount of work
+ * @param abort if not 0, can be set to true to abort operation
+ */
+void BaseMainWindowImpl::showOperationProgress(const QString& name,
+                                               int done, int total, bool* abort)
+{
+  if (done == -1) {
+    // Operation started.
+    if (!m_progressBar) {
+      m_progressBar = new QProgressBar;
+    }
+    if (!m_progressAbortButton) {
+      m_progressAbortButton = new QToolButton;
+      m_progressAbortButton->setIcon(
+            QIcon(m_w->style()->standardIcon(QStyle::SP_BrowserStop)));
+      m_progressAbortButton->setToolTip(tr("Abort"));
+      m_progressAbortButton->setCheckable(true);
+    }
+    m_w->statusBar()->addPermanentWidget(m_progressBar);
+    m_w->statusBar()->addPermanentWidget(m_progressAbortButton);
+    m_progressBar->setMinimum(0);
+    m_progressBar->setMaximum(total);
+    m_progressBar->setValue(0);
+    m_progressAbortButton->setChecked(false);
+    slotStatusMsg(name);
+  } else if (done == total && total != 0) {
+    // Operation finished.
+    if (m_progressBar) {
+      m_w->statusBar()->removeWidget(m_progressBar);
+      delete m_progressBar;
+      m_progressBar = 0;
+    }
+    if (m_progressAbortButton) {
+      m_w->statusBar()->removeWidget(m_progressAbortButton);
+      delete m_progressAbortButton;
+      m_progressAbortButton = 0;
+    }
+    slotStatusMsg(tr("Ready."));
+  } else if (done < total || (done == 0 && total == 0)) {
+    // Operation progress.
+    if (m_progressBar) {
+      m_progressBar->setMaximum(total);
+      m_progressBar->setValue(done);
+      // Is needed to get abort button events.
+      qApp->processEvents();
+    }
+    if (m_progressAbortButton && m_progressAbortButton->isChecked() && abort) {
+      *abort = true;
+    }
+  }
+}
+
 /**
  * Save all changed files.
  *
@@ -240,18 +302,8 @@ void BaseMainWindowImpl::saveDirectory(bool updateGui)
   if (updateGui) {
     updateCurrentSelection();
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-    slotStatusMsg(tr("Saving directory..."));
   }
 
-  QProgressBar* progress = new QProgressBar;
-  m_w->statusBar()->addPermanentWidget(progress);
-  progress->setMinimum(0);
-  connect(m_app, SIGNAL(saveStarted(int)),
-          progress, SLOT(setMaximum(int)));
-  connect(m_app, SIGNAL(saveProgress(int)),
-          progress, SLOT(setValue(int)));
-  qApp->processEvents();
-
 #ifdef Q_OS_WIN32
   // Close player on Windows because it holds file handles which prevent
   // files from being saved.
@@ -265,8 +317,6 @@ void BaseMainWindowImpl::saveDirectory(bool updateGui)
 
   QStringList errorFiles = m_app->saveDirectory();
 
-  m_w->statusBar()->removeWidget(progress);
-  delete progress;
   if (!errorFiles.empty()) {
     QStringList errorMsgs, notWritableFiles;
     foreach (const QString& filePath, errorFiles) {
@@ -308,7 +358,6 @@ void BaseMainWindowImpl::saveDirectory(bool updateGui)
   }
 
   if (updateGui) {
-    slotStatusMsg(tr("Ready."));
     QApplication::restoreOverrideCursor();
     updateGuiControls();
   }
@@ -815,8 +864,10 @@ void BaseMainWindowImpl::slotFilter()
       m_filterDialog = new FilterDialog(m_w);
       connect(m_filterDialog, SIGNAL(apply(FileFilter&)),
               m_app, SLOT(applyFilter(FileFilter&)));
-      connect(m_app, SIGNAL(fileFiltered(int,QString)),
+      connect(m_app, SIGNAL(fileFiltered(int,QString,int,int)),
               m_filterDialog, SLOT(showFilterEvent(int,QString)));
+      connect(m_app, SIGNAL(fileFiltered(int,QString,int,int)),
+              this, SLOT(filterProgress(int,QString,int,int)));
     }
     FilterConfig::instance().setFilenameFormat(
           FileConfig::instance().toFilenameFormat());
@@ -825,6 +876,42 @@ void BaseMainWindowImpl::slotFilter()
   }
 }
 
+/**
+ * Show filter operation progress.
+ * @param type filter event type
+ * @param fileName name of file processed
+ * @param passed number of files which passed the filter
+ * @param total total number of files checked
+ */
+void BaseMainWindowImpl::filterProgress(int type, const QString& fileName,
+                                        int passed, int total)
+{
+  Q_UNUSED(fileName)
+  switch (type) {
+  case FileFilter::Started:
+    startProgressMonitoring(tr("Filter"), &BaseMainWindowImpl::terminateFilter,
+                            true);
+    break;
+  case FileFilter::Finished:
+  case FileFilter::Aborted:
+    stopProgressMonitoring();
+    break;
+  default:
+    checkProgressMonitoring(0, 0, QString::number(passed) +
+                            QLatin1Char('/') + QString::number(total));
+  }
+}
+
+/**
+ * Terminate filtering the file list.
+ */
+void BaseMainWindowImpl::terminateFilter()
+{
+  if (m_filterDialog) {
+    m_filterDialog->abort();
+  }
+}
+
 #if defined HAVE_PHONON || QT_VERSION >= 0x050000
 /**
  * Play audio file.
@@ -845,8 +932,10 @@ void BaseMainWindowImpl::showPlayToolBar()
     m_w->addToolBar(Qt::BottomToolBarArea, m_playToolBar);
     connect(m_playToolBar, SIGNAL(errorMessage(QString)),
             this, SLOT(slotStatusMsg(QString)));
+#ifdef HAVE_QTDBUS
     connect(m_playToolBar, SIGNAL(closed()),
             m_app, SLOT(deactivateMprisInterface()));
+#endif
 #ifdef Q_OS_WIN32
     // Phonon on Windows cannot play if the file is open.
     connect(m_playToolBar, SIGNAL(aboutToPlay(QString)),
@@ -1203,7 +1292,6 @@ void BaseMainWindowImpl::toggleExpanded(const QModelIndex& index)
 void BaseMainWindowImpl::expandFileList()
 {
   m_expandNotificationNeeded = sender() == m_app;
-  m_expandFileListStartTime = QDateTime::currentDateTime();
   connect(m_app->getFileProxyModelIterator(),
           SIGNAL(nextReady(QPersistentModelIndex)),
           this, SLOT(expandNextDirectory(QPersistentModelIndex)));
@@ -1212,8 +1300,12 @@ void BaseMainWindowImpl::expandFileList()
   QObject* emitter = sender();
   bool sentFromAction =
       emitter && emitter->metaObject() == &QAction::staticMetaObject;
-  m_app->getFileProxyModelIterator()->start(
-        sentFromAction && QApplication::keyboardModifiers() == Qt::ShiftModifier
+  bool expandOnlySubtree =
+      sentFromAction && QApplication::keyboardModifiers() == Qt::ShiftModifier;
+  startProgressMonitoring(tr("Expand All"),
+                          &BaseMainWindowImpl::terminateExpandFileList,
+                          !expandOnlySubtree);
+  m_app->getFileProxyModelIterator()->start(expandOnlySubtree
         ? m_form->getFileList()->currentIndex()
         : m_form->getFileList()->rootIndex());
 }
@@ -1225,41 +1317,115 @@ void BaseMainWindowImpl::expandFileList()
  */
 void BaseMainWindowImpl::expandNextDirectory(const QPersistentModelIndex& index)
 {
-  bool terminated = !index.isValid();
-  if (!terminated) {
+  if (index.isValid()) {
     if (m_app->getFileProxyModel()->isDir(index)) {
       m_form->getFileList()->expand(index);
     }
-    if (m_expandFileListStartTime.isValid() &&
-        m_expandFileListStartTime.secsTo(QDateTime::currentDateTime()) >= 3) {
-      // Operation is taking some time, show dialog to abort it.
-      m_expandFileListStartTime = QDateTime();
-      if (!m_progressDialog) {
-        m_progressDialog = new QProgressDialog(m_w);
-      }
-      m_progressDialog->setWindowTitle(tr("Expand All"));
-      m_progressDialog->setLabelText(QString());
-      m_progressDialog->setCancelButtonText(tr("A&bort"));
-      m_progressDialog->setMinimum(0);
-      m_progressDialog->setMaximum(0);
-      m_progressDialog->setAutoClose(true);
-      m_progressDialog->show();
-    }
-    if (m_progressDialog && m_progressDialog->wasCanceled()) {
-      terminated = true;
+    int done = m_app->getFileProxyModelIterator()->getWorkDone();
+    int total = m_app->getFileProxyModelIterator()->getWorkToDo() + done;
+    checkProgressMonitoring(done, total, QString());
+  } else {
+    stopProgressMonitoring();
+  }
+}
+
+/**
+ * Terminate expanding the file list.
+ */
+void BaseMainWindowImpl::terminateExpandFileList()
+{
+  m_app->getFileProxyModelIterator()->abort();
+  disconnect(m_app->getFileProxyModelIterator(),
+             SIGNAL(nextReady(QPersistentModelIndex)),
+             this, SLOT(expandNextDirectory(QPersistentModelIndex)));
+  if (m_expandNotificationNeeded) {
+    m_expandNotificationNeeded = false;
+    m_app->notifyExpandFileListFinished();
+  }
+}
+
+/**
+ * Start monitoring the progress of a possibly long operation.
+ *
+ * If the operation takes longer than 3 seconds, a progress widget is shown.
+ *
+ * @param title title to be displayed in progress widget
+ * @param terminationHandler method to be called to terminate operation
+ * @param disconnectModel true to disconnect the file list models while the
+ * progress widget is shown
+ */
+void BaseMainWindowImpl::startProgressMonitoring(const QString& title,
+                                                 void (BaseMainWindowImpl::*terminationHandler)(),
+                                                 bool disconnectModel)
+{
+  if (!m_progressTitle.isEmpty() && m_progressTitle != title) {
+    stopProgressMonitoring();
+  }
+  m_progressTitle = title;
+  m_progressTerminationHandler = terminationHandler;
+  m_progressDisconnected = disconnectModel;
+  m_progressStartTime = QDateTime::currentDateTime();
+}
+
+/**
+ * Start monitoring the progress started with startProgressMonitoring().
+ */
+void BaseMainWindowImpl::stopProgressMonitoring()
+{
+  if (m_progressWidget) {
+    m_form->removeLeftSideWidget(m_progressWidget);
+    m_progressWidget->reset();
+    if (m_progressDisconnected) {
+      m_form->getDirList()->reconnectModel();
+      m_form->getFileList()->reconnectModel();
+      m_form->getFileList()->expandAll();
     }
   }
-  if (terminated) {
-    m_app->getFileProxyModelIterator()->abort();
-    disconnect(m_app->getFileProxyModelIterator(),
-               SIGNAL(nextReady(QPersistentModelIndex)),
-               this, SLOT(expandNextDirectory(QPersistentModelIndex)));
-    if (m_progressDialog) {
-      m_progressDialog->reset();
+  if (m_progressTerminationHandler) {
+    (this->*m_progressTerminationHandler)();
+  }
+  m_progressTitle.clear();
+  m_progressTerminationHandler = 0;
+}
+
+/**
+ * Check progress of a possibly long operation.
+ *
+ * Progress monitoring is started with startProgressMonitoring(). This method
+ * will check if the opeation is running long enough to show a progress widget
+ * and update the progress information. It will call stopProgressMonitoring()
+ * when the operation is aborted.
+ *
+ * @param done amount of work done
+ * @param total total amount of work
+ * @param text text for progress label
+ */
+void BaseMainWindowImpl::checkProgressMonitoring(int done, int total,
+                                                 const QString& text)
+{
+  if (m_progressStartTime.isValid() &&
+      m_progressStartTime.secsTo(QDateTime::currentDateTime()) >= 3) {
+    // Operation is taking some time, show dialog to abort it.
+    m_progressStartTime = QDateTime();
+    if (!m_progressWidget) {
+      m_progressWidget = new ProgressWidget(m_w);
     }
-    if (m_expandNotificationNeeded) {
-      m_expandNotificationNeeded = false;
-      m_app->notifyExpandFileListFinished();
+    m_progressWidget->setWindowTitle(m_progressTitle);
+    m_progressWidget->setLabelText(QString());
+    m_progressWidget->setCancelButtonText(tr("A&bort"));
+    m_progressWidget->setMinimum(0);
+    m_progressWidget->setMaximum(0);
+    m_form->setLeftSideWidget(m_progressWidget);
+    if (m_progressDisconnected) {
+      m_form->getFileList()->disconnectModel();
+      m_form->getDirList()->disconnectModel();
+    }
+  }
+  if (m_progressWidget) {
+    m_progressWidget->setValueAndMaximum(done, total);
+    m_progressWidget->setLabelText(text);
+    if (m_progressWidget->wasCanceled()) {
+      stopProgressMonitoring();
     }
   }
 }
